
// #include "io430.h"
#include <in430.h>
#include <msp430f2274.h>
#include <stdbool.h>
#include <stddef.h>

//#include "i2c.h"
#include "hardware.h"
#include "commanddefs.h"
#include "470x.h"
#include "propertydefs.h"
#include "FMRXrds.h"
#include "flash.h"

//-----------------------------------------------------------------------------
// Defines
//-----------------------------------------------------------------------------
#define POWERUP_TIME 250    	// Powerup delay in milliseconds
#define POWER_UP    0x01
#define CTL_WRITE   0x48		//fr SPI BUS SDIO schreiben
#define CTL_READ1   0xA0		//fr SPI BUS GPOI READ 1 Byte
#define CTL_READ16  0xE0		//fr SPI BUS GPOI READ 16 Byte
#define IO3W    0
#define IO2W    1
#define IOSPI   2
//When SEN = 0, the seven-bit device address is 0010001b. When SEN = 1, the address is 1100011b.
#define IO2W_SENB0_ADDRESS 0x22 //0x11 um 1 bit verschoben!
#define IO2W_SENB1_ADDRESS 0xC6	//0x63 um 1 bit verschoben! 
#define READ    1
#define WRITE   0
#define GPO_IEN				 0x0001
//#define DEMO_DELAY 5000
#define BLER_SCALE_MAX 200  // Block Errors are reported in .5% increments

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
//extern bool WaitSTCInterrupt;
extern bool PoweredUp;
extern bool SeekTuneInProc;
u8 chipFunction;
unsigned char io_mode = IO3W; //IOSPI; //IO2W; // IO3W;
bool SeekTuneInProc = 0;
bool WaitSTCInterrupt = 0;
char STC; //xdata 
char BLTF; //xdata 
char AFCRL; //xdata 
char Valid; //xdata 
int Freq; //xdata 
char RSSI; //xdata 
char ASNR; //xdata 
int AntCap; //xdata 
u8 cmd[8]; //  idata   9 wegen SPI (sonst 8)
u8 rsp[16]; //  idata 
bool RdsTestInProc = 0;
unsigned char RdsAvailable = 0;    //idata 
bool PoweredUp = 0;
unsigned char ProcessSame = 0;     // idata 
bool SameTestInProc = 0;
char Status;   //xdata
unsigned char Alert; // extern xdata 
unsigned char AsqInts;//extern xdata

//-----------------------------------------------------------------------------
// Function prototypes
//-----------------------------------------------------------------------------
void si47xx_getPartInformation(void);;
unsigned char getIntStatus(void);
static void die(void);

void updateRds(void);

void fmTuneFreq(int frequency);
char fmTuneStatus(unsigned char cancel, unsigned char intack);
void fmRdsStatus(unsigned char intack, unsigned char mtfifo);
extern void si47_rdsGetBler(int *bler);

void test_FMTXrds(void);
void si47xxFMRX_initialize(void);
void si47xxFMRX_powerup(void);
void si47xxFMRX_set_volume(unsigned char volume);
void si47xxFMRX_configure(void);
static void si47xxFMRX_general_cfg(void);
static void si47xxFMRX_hardware_cfg(void);
unsigned char si47xxFMRX_tune(int frequency);

static void display_PCH();	//bpark
static void testlauf2(u8 *byte);

void LED();

//Pat
void SCL_low(void);
void SCL_high(void);
void SCL_clock(void);	//HL-Sequenz
void SDA_low(void);
void SDA_high(void);
void SDA_read(void);
void SDA_write(void);
void SCL_write(void);
void SCL_read(void);
void SDA_OUT(bool Data);
bool SDA_IN();
void SEN_low();			//RDS-SEN-Pin
void SEN_high();		//RDS-SEN-Pin	
 
//-----------------------------------------------------------------------------

void si47xx_reset()
//-----------------------------------------------------------------------------
////P2.0  RST
// 0x80 GPIO_CTL2 Configures GPO1, 2, and 3 as output or Hi-Z.
// 0x81 GPIO_SET2 Sets GPO1, 2, and 3 output level (low or high). 
//GPIO feature (command GPIO_CTL and GPIO_SET) is fully supported in FMTX component 3.0 or higher.
//It is partially supported (GPO3 only) in FMTX component 2.0.
// Based on what IO mode is being used configure GPIO1 and GPIO2
{
	//Table 20. Bus Mode Selection
	//Bus Mode GPO2/INT GPO1
	//2-wire     0        1
	   
	//SEN_low();
	P2SEL &= 0x00;
	P2DIR |= 0x06;  
	P2DIR |= 0x06;
	P2REN &= 0xF9;
	   
	switch (io_mode)
	{
		
		case IOSPI:		//SPI 1 (must drive)   2
		{
			//P2.1 und P2.2 auf 1
	        P2OUT |= 0x06;	 
	   		break;
		}
	   
	  	case IO2W:	//if(io_mode == IO2W)	1
		{
	    	P2OUT |= 0x04;
	    
	    	P2OUT &=0xFD;
	    	break;
		}
		
		case IO3W:	//0
		{
	    	P2OUT &=0xFB;
	    	P2OUT &=0xFD;
	  		break;
		}
	}
	  
	//Reset P2.0, um Busmode zu setzten
	P2SEL &= 0x00;
	P2REN |= 0x01;
	P2DIR |= 0x01;
	wait_us(10);
	P2OUT &= 0xFE;
	wait_us(10);
	P2OUT |= 0x01;
	wait_us(10);
	//GPO-Defaults -> hier als Input
	//GPO1        P2.2
	//GPO2/INT    P2.1
	P2SEL &= 0x00;
	P2REN |= 0x06;
	P2OUT &= 0xF9;	//vorsichtshalber auf 0
	P2DIR &= 0xF9;
	 
}

//-----------------------------------------------------------------------------
// This command returns the status
//-----------------------------------------------------------------------------
unsigned char si47xx_readStatus()
{
    u8 status;
   
    si47xx_lowRead(1, &status);

    return status;
}

//-----------------------------------------------------------------------------
// Command that will wait for CTS before returning
//-----------------------------------------------------------------------------
int si47xx_waitForCTS()
{
    int i=100;
    unsigned char rStat = 0x00;
	
	rStat = si47xx_readStatus();	
    // Loop until CTS is found or stop due to the counter running out.
    while (--i && !(rStat & CTS))
    {
        rStat = si47xx_readStatus();
    }
    
   	return 100-i;
}

//-----------------------------------------------------------------------------
// Sends a command to the part and returns the reply bytes
//-----------------------------------------------------------------------------
void si47xx_command(u8 cmd_size, u8 *cmd, u8 reply_size, u8 *reply)
{
    // It is always a good idea to check for cts prior to sending a command to
    // the part.

	switch (io_mode)
		{
			case IOSPI:	//2
			{
				SEN_low();
				//wait_us(5);

			    // Write the command to the part
			    si47xx_lowWrite(cmd_size, cmd);
			
			    // If the calling function would like to have results then read them.
			     if(reply_size)
			    {    	
			    	SEN_high();
			    		
			    	SEN_low();
			    		
			        si47xx_lowRead(reply_size, reply);
			    }
			    
			    SEN_high();
			    break;
			}
			
			case IO3W:	//0
    		{
    			//SEN_low();

			    // Write the command to the part
			    si47xx_lowWrite(cmd_size, cmd);
			
			    // If the calling function would like to have results then read them.
			     if(reply_size)
			    {    	
			    	//SEN_low();		    		
			        si47xx_lowRead(reply_size, reply);
			    }
			    break;
			}
		}
}

//-----------------------------------------------------------------------------
// Helper function that is used to write to the part which abstracts what
// bus mode is currently being used.
//-----------------------------------------------------------------------------
void si47xx_lowWrite(u8 number_bytes, u8 *data_out)
{
	
		switch (io_mode)
		{
		
			case IOSPI:	//2
			{
				//ioSPI_write(number_bytes, data_out);
				break;
			}
				
			case IO3W:	//0
    		//if(io_mode == IO3W)
    		{
        		io3w_write(number_bytes, data_out);
    			break;
    		}
    		
    		case IO2W:	//1
    		{ 
        		//io2w_write(number_bytes, data_out);
        		break;
    		}
		}
}

//-----------------------------------------------------------------------------
// Helper function that is used to read from the part which abstracts what
// bus mode is currently being used.
//-----------------------------------------------------------------------------
void si47xx_lowRead(u8 number_bytes, u8  *data_in)
{
	
	switch (io_mode)
	{
		
		case IOSPI:	//2
			//ioSPI_read(number_bytes, data_in);
			break;
	
    	case IO3W:	//if(io_mode == IO3W)	0
    	{
       		io3w_read(number_bytes, data_in);
       		break;
    	}
    		
    		
    	case IO2W:	//else // IO2W	//1
    	{
        	//io2w_read(number_bytes, data_in);
        	break;
    	}
    		
	}
}

//-----------------------------------------------------------------------------
// Returns the appropriate address of the device based on the SENB state
//
// Outputs:
//      The address of the part
//-----------------------------------------------------------------------------
unsigned char io2w_get2wAddress()
{
    unsigned char addr;
    //if(SENB) {  // Routed to SENB on the Si47xx Part
    //    addr = IO2W_SENB1_ADDRESS;
   //} else {
        addr = IO2W_SENB0_ADDRESS;
   // }
    return addr;
}

//-----------------------------------------------------------------------------
// Sends 2-wire start, writes an array of data, sends 2-wire stop.
//
// Inputs:
//		number_bytes: The number of bytes to write
//		data_out:     Source array for data to be written
//
//-----------------------------------------------------------------------------
void io2w_write(u8 number_bytes, u8 *data_out)
{
    u8 addr;

    // issue the START condition with address lsb cleared for writes
	addr = io2w_get2wAddress();
    io2w_start(addr, WRITE);

    // loop writing all bytes in the data_out array
    while(number_bytes--)
    {
        io2w_write_byte(*data_out++);
    }

    // issue the STOP condition
    io2w_stop();
}

//-----------------------------------------------------------------------------
// Sends 2-wire start, reads an array of data, sends 2-wire stop.
//
// Inputs:
//		number_bytes: Number of bytes to be read
//		data_in:      Destination array for data read
//
//-----------------------------------------------------------------------------
void io2w_read(u8 number_bytes,u8 *data_in)
{
    u8 addr;
	
	//SEN_low();
    // issue the START condition with address lsb set for reads
	addr = io2w_get2wAddress();
    io2w_start(addr, READ);

    // loop until the specified number of bytes have been read
    while(number_bytes--)
    {
        *data_in++ = io2w_read_byte(number_bytes);
    }

    // issue the STOP condition
    io2w_stop();
}

//-----------------------------------------------------------------------------
// Send a 2-wire start and address byte. LSB will be set to indicate a read,
// cleared to indicate a write.
//
// A START condition is defined as a high to low transition on the SDIO pin
// while SCLK is high.
//
// Inputs:
//      io2w_address:  Address of device to access.
//      operation: Set to 1 for a read, set to 0 for a write
//
// Pin drive status upon exit:
//      SDIO = high (open-drain input)
//      SCLK = high
//-----------------------------------------------------------------------------
static void io2w_start(char io2w_address, char operation)
{
    int i;
		
	SCL_high(); //SCLK = 1;
    
    // PORT 0 OUTPUT MODE 
	//SDIO_OUT |= SDIO_HEX;  // Configure SDIO as push-pull
    //P3.2 
    P3OUT |= 0x04;
    
    // issue the START condition
    SDA_low(); //SDIO = 0;
    SCL_low();  //SCLK = 0;

    // Set the least significant bit to indicate read or write (read = 1, write = 0).
    io2w_address = (io2w_address & 0xFE) | (operation & 0x01);

    // issue the control word (7 bit chip address + R/W* bit)
    // Note that tr:IN + tLOW + tf:IN + tHIGH = 2500 ns = 400 kHz
    for ( i = 7; i >= 0; i-- )
    {
        SCL_low(); //SCLK = 0;
        //wait_us(1); // tf:IN
        SDA_OUT((io2w_address >> i) & 0x01);  //SDIO = ((io2w_address >> i) & 0x01);
        //wait_us(1); // tLOW
        SCL_high();  //SCLK = 1;
        //wait_us(1); // tf:IN + tHIGH
    }

    // check the acknowledge
		//SDIO_OUT &= ~(SDIO_HEX);   // Configure SDIO as open-drain
    //P3.2 als Input
    //P3OUT &= 0x0B; //test
    P3DIR &= 0xFB;
    P3REN &= 0xFB;    //PullUp-einschalten
    SCL_low(); //SCLK = 0;
    //hier mu dann ACK auf low gehen
    
	if (SDA_IN() != 0)
        die();  // ack not received.  This should never happen. Device isn't responding.
}

//-----------------------------------------------------------------------------
// Write one byte of data.
// 
// Inputs:
//		wrdata:  Byte to be written
//
// Pin drive status upon exit:
//      SDIO = high (open-drain input)
//      SCLK = high
//-----------------------------------------------------------------------------
static void io2w_write_byte(char wrdata)
{
    unsigned char i;

    SDA_write();
    for (i=7;i>=0;i--)
    {
        SCL_low(); //SCLK = 0;
        wait_us(1); // tf:IN
        SDA_OUT(((wrdata >> i) & 0x01));   //SDIO = ((wrdata >> i) & 0x01);
        wait_us(2); // tLOW
        SCL_high(); //SCLK = 1;
        wait_us(1); // tf:IN + tHIGH
    }
    
    // check the acknowledge
	P3REN &= 0xFB;
	SDA_read();   //SDIO_OUT &= ~(SDIO_HEX);  // Configure SDIO as open-drain
    SCL_low(); //SCLK = 0;
    SDA_high(); //SDIO = 1;   // Configure P0^7(SDIO) as a digital input
    wait_us(2); // tf:IN + tLOW
    SCL_high(); //SCLK = 1;
    //wait_us(1); // tf:IN + tHIGH

    if (SDA_IN() != 0) //(SDIO != 0)
        die();  // ack not received.  This should never happen. Device isn't responding.
}

//-----------------------------------------------------------------------------
// Send a 2-wire stop.  A STOP condition is defined as a low to high transition
// on the SDIO pin while SCLK is high.
//
// Pin drive status upon exit:
//      SDIO = high (open-drain input)
//      SCLK = high
//-----------------------------------------------------------------------------
static void io2w_stop(void)
{
    SDA_write();  //SDIO_OUT |= SDIO_HEX;  // Configure SDIO as push-pull
	SCL_low(); //SCLK = 0;
    wait_us(2); // tf:IN + tLOW
    SDA_low();  //SDIO = 0;
    wait_us(1);
    SCL_high(); //SCLK = 1;
    wait_us(1); // tf:IN + tSU:STO
    SDA_high();  //SDIO = 1;
}

//-----------------------------------------------------------------------------
// Read one byte of data.
//
// Inputs:
//		remaining bytes:  Zero value indicates an ack will not be sent.
//
// Outputs:
// 		Returns byte read
//
// Pin drive status upon exit:
//      SDIO = high (open-drain input) if no more bytes to be read
//      SDIO = low (open-drain input) if read is to continue
//      SCLK = high
//-----------------------------------------------------------------------------
static char io2w_read_byte(char remaining_bytes)
{
    unsigned char i;
    unsigned char rddata = 0;

    P3REN &= 0xFB; //SDIO_OUT &= ~(SDIO_HEX);  // Configure SDIO as open-drain
    SDA_read();
    for( i = 7; i >= 0; i-- )
    {
        SCL_low(); //SCLK = 0;
          //SDIO = 1;                        // Configure P0^7(SDIO) as a digital input
        wait_us(10);                      // tf:IN
        SCL_high(); //SCLK = 1;
        wait_us(10);                      // tf:IN + tHIGH
        rddata = ((rddata << 1) | SDA_IN());  // rddata = ((rddata << 1) | SDIO);
    }
    // set the acknowledge
    SCL_low(); //SCLK = 0;

    //SDA_write();    //SDIO_OUT |= SDIO_HEX;   // Configure SDIO as push-pull
		P3REN |= 0x04;	
    if (remaining_bytes == 0)
        SDA_high();  //SDIO = 1;
    else
        SDA_low();  //SDIO = 0;

    wait_us(10); // tf:IN + tLOW
    SCL_high(); //SCLK = 1;
    wait_us(10); // tf:IN + tHIGH

    return rddata;
}

//-----------------------------------------------------------------------------
// Resets the part and initializes registers to the point of being ready for
// the first tune or seek.
//-----------------------------------------------------------------------------
void si47xxInit(void)
{
    // Zero status registers.
	PoweredUp = 0;

    // Perform a hardware reset, power up the device, and then perform the
    // initial configuration.
    si47xx_reset();
    si47xxFMRX_powerup();
    
}

//-----------------------------------------------------------------------------
// Set the passed property number to the passed value.
//
// Inputs:
//      propNumber:  The number identifying the property to set
//      propValue:   The value of the property.
//-----------------------------------------------------------------------------
void si47xx_set_property(int propNumber, int propValue)
{	
	unsigned char i;
	
    // Put the ID for the command in the first byte.
    cmd[0] = SET_PROPERTY;	//0x12

	// Initialize the reserved section to 0
    cmd[1] = 0;

	// Put the property number in the third and fourth bytes.
    cmd[2] = (unsigned char)(propNumber >> 8);
	cmd[3] = (unsigned char)(propNumber & 0x00FF);

	// Put the property value in the fifth and sixth bytes.
    cmd[4] = (unsigned char)(propValue >> 8);
    cmd[5] = (unsigned char)(propValue & 0x00FF);

	  for (i=6; i<8; i++ )
    {
    	cmd[i] = 0x00;
    }

    // Invoke the command
	si47xx_command(8, cmd, 0, NULL);
}

//-----------------------------------------------------------------------------
// Set the passed property number to the passed value.
//
// Inputs:
//      propNumber:  The number identifying the property to set
//      propValue:   The value of the property.
//-----------------------------------------------------------------------------
int si47xx_get_property(int propNumber)
{	
	unsigned char i;
	int wert;

    // Put the ID for the command in the first byte.
    cmd[0] = GET_PROPERTY;	//0x13

	// Initialize the reserved section to 0
    cmd[1] = 0x00;

	// Put the property number in the third and fourth bytes.
    cmd[2] = (unsigned char)(propNumber >> 8);
	cmd[3] = (unsigned char)(propNumber & 0x00FF);

	  for (i=4; i<8; i++ )
    {
    	cmd[i] = 0x00;
    }

    // Invoke the command
	si47xx_command(8, cmd, 16, rsp);
	
	wert  = (int)(rsp[2] << 8) | (int)rsp[3]; 
	
	return wert;
}

//-----------------------------------------------------------------------------
// This is used just for debugging to detect when the si47xx is not responding.
// The 47xx should always ack 2-wire transactions.  If it does not, there is a
// hardware problem that must be resolved.
//-----------------------------------------------------------------------------
static void die(void)
{
    _nop_(); // put breakpoint here during debug.
}

//-----------------------------------------------------------------------------
// Helper function that sends the GET_INT_STATUS command to the part
//
// Returns:
//   The status byte from the part.
//-----------------------------------------------------------------------------
unsigned char getIntStatus(void)
{
	unsigned char i;

    // Put the ID for the command in the first byte.
    cmd[0] = GET_INT_STATUS; //0x14
	
	  for (i=1; i<8; i++) 
  	{
    	cmd[i] = 0x00;
  	}		

    // Invoke the command
	si47xx_command(8, cmd, 16, rsp);

	// Return the status
	return rsp[0];
}

// Fr den debugger/Breakpoints - by wolf
void _nop_(void)
{
}

//-----------------------------------------------------------------------------
// Interrupt service routine for RDS and STC
//-----------------------------------------------------------------------------
void GPIO2_ISR(void) // interrupt 0
{
	if(PoweredUp)
	{
		// This interrupt can be fired by either RDS or STC.
	    if (SeekTuneInProc)
	    {
	        // It was a STC interrupt
	        WaitSTCInterrupt = 0;
	    }
		else if(RdsTestInProc)
		{
			// Increment the RDS available flag
			RdsAvailable++;
		}
		else if(SameTestInProc)
		{
			// Increment the Process SAME flag
			ProcessSame++;
		}
	}
}

//-----------------------------------------------------------------------------
// Resets the part and initializes registers to the point of being ready for
// the first tune or seek.
//-----------------------------------------------------------------------------
void si47xxFMRX_initialize(void)
{
    // Zero status registers.
	PoweredUp = 0;

    // Perform a hardware reset, power up the device, and then perform the
    // initial configuration.
    si47xx_reset();
    
    switch (io_mode)
    {
    	case IO3W:	//0
    	{
    		si47xxFMRX_powerup();
    		break;
    	}
    		
    	case IOSPI:	//2
    	{
    		break;
    	}
    }

}

//-----------------------------------------------------------------------------
// Tunes to a station number using the current band and spacing settings.
//
// Inputs:
//      frequency:  frequency in 10kHz steps
//
// Returns:
//      The RSSI level found during the tune.
//-----------------------------------------------------------------------------
unsigned char si47xxFMRX_tune(int frequency)
{
	// Enable the bit used for the interrupt of STC.
	SeekTuneInProc = 1;

	// Call the tune command to start the tune.
 	WaitSTCInterrupt = 1;
    
    fmTuneFreq(frequency);
	
    // wait for the interrupt before continuing
    // If you do not wish to use interrupts but wish to poll the part
    // then comment out this line.
    //while (WaitSTCInterrupt); // Wait for interrupt to clear the bit

    // Wait for stc bit to be set
    while (!(getIntStatus() & STCINT));

	// Clear the STC bit and get the results of the tune.
	fmTuneStatus(0, 1); //(CANCEL, INTACK)

	// Disable the bit used for the interrupt of STC.
	SeekTuneInProc = 0;

    // Return the RSSI level
    return RSSI;
}

//-----------------------------------------------------------------------------
// Take the Si47xx out of powerdown mode.
//-----------------------------------------------------------------------------
void si47xxFMRX_powerup(void)
{
	unsigned char i;

    // Check if the device is already powered up.
    if (PoweredUp)
    {
    }
    else 
    {        
        //cmd[0] = CTL_WRITE; //write
        
        cmd[0] = POWER_UP; //0x01

        //cmd[1] = 0x00; //0x90;	//0x00;
		//cmd[1] =  0xC0; //Set to FM Receive. Enable interrupts.
		//cmd[1] =  0x90; //Set XOSCEN Use crystal oscillator (RCLK and GPO3/DCLK with external 32.768 kHz crystal
						//and OPMODE=00000101) //wird im testbericht von Alpha Board gemacht
		//cmd[1] = 0x40;	//POWER_UP_IN_GPO2OEN
		
		cmd[1] =  0xD0; //Set to FM Receive. Enable interrupts. + ext. Osc.
		
		// The opmode needs to be set to analog mode
        cmd[2] = POWER_UP_IN_OPMODE_RX_ANALOG; //0x05 Analog Audio Output
        
	    //unused arguments must be 0x00 (SPI)  
	    for (i=3; i<8; i++)
	    {
	    	cmd[i] = 0x00;
	    }

        // Powerup the device
		si47xx_command(8, cmd, 0, rsp);
        wait_ms(POWERUP_TIME);               // wait for si47xx to powerup

        // Since we did not boot the part in query mode the result will not
        // contain the part information.

		PoweredUp = 1;
    }
}

//-----------------------------------------------------------------------------
// Set the volume and mute/unmute status
//
// Inputs:
//      volume:    a 6-bit volume value
//
// Note: It is assumed that if the volume is being adjusted, the device should
// not be muted.
//-----------------------------------------------------------------------------
void si47xxFMRX_set_volume(unsigned char volume)
{
    switch (io_mode)
    {
    	case IOSPI:
    	{
//    		// Turn off the mute
//    		si47xx_SPIset_property(RX_HARD_MUTE, 0);
//
//    		// Set the volume to the passed value
//    		si47xx_SPIset_property(RX_VOLUME, (int)volume & RX_VOLUME_MASK);
    	}
    	
    	case IO3W:
    	{
    		// Turn off the mute
    		si47xx_set_property(RX_HARD_MUTE, 0);

    		// Set the volume to the passed value
    		si47xx_set_property(RX_VOLUME, (int)volume & RX_VOLUME_MASK);
    	}
	}
}

//-----------------------------------------------------------------------------
// Compute the block error rate.
// Returns a value between 0 and BLER_SCALE_MAX (200) which can be used
// to determine the % errors in 0.5% resolution.
//-----------------------------------------------------------------------------
void si47_rdsGetBler(int *bler)
{
    if (RdsBlocksTotal < RdsBlocksValid)
    {
        // The timer which generates the total block count is not synchronized
        // to the RDS signal, so occasionally it looks like we received more
        // valid blocks than we should have. In this case, just assume no
        // errors.
        *bler = 0;
    }
    else if (!RdsBlocksTotal)
    {
        // If the timer has not fired, we will assume 100% errors
        *bler = BLER_SCALE_MAX;
    }
    else
    {
        // Calculate the block error rate
        *bler = BLER_SCALE_MAX - ((RdsBlocksValid * BLER_SCALE_MAX) / RdsBlocksTotal);
    }
}


//-----------------------------------------------------------------------------
// After tuning to a new station, the RDS tracking variables need to be
// initialized.
//-----------------------------------------------------------------------------
void initRdsVars(void)
{
    u8 i;
	
    // Set the maximum allowable errors in a block considered acceptable
    // It's critical that block B be correct since it determines what's
    // contained in the latter blocks. For this reason, a stricter tolerance
    // is placed on block B
    if (!firstInitDone)
    {
        rdsBlerMax[0] = CORRECTED_THREE_TO_FIVE; // Block A
        rdsBlerMax[1] = CORRECTED_ONE_TO_TWO;    // Block B
//        rdsBlerMax[2] = CORRECTED_THREE_TO_FIVE; // Block C
//        rdsBlerMax[3] = CORRECTED_THREE_TO_FIVE; // Block D
        rdsBlerMax[2] = 0; // Block C
        rdsBlerMax[3] = 0; // Block D
        firstInitDone = 1;
    }

    // Reset RDS variables
    RdsDataAvailable = 0;
    RdsDataLost      = 0;
    RdsIndicator     = 0;
    RdsBlocksValid   = 0;
    RdsBlocksTotal   = 0;

    // Clear RDS Fifo
    RdsFifoEmpty = 1;
    
    RdsValid[0]    = 0;
    RdsValid[1]    = 0;
    RdsValid[2]    = 0;
    RdsValid[3]    = 0;
    RdsGroups      = 0;

    LED(0);
    
}

//-----------------------------------------------------------------------------
// After an RDS interrupt, read back the RDS registers and process the data.
//-----------------------------------------------------------------------------
void updateRds(void)
{
    u8 PCHblocks[4];
    u8 group_type;      // bits 4:1 = type,  bit 0 = version
    u8 errorCount;
    bool abflag=0;

    RdsDataAvailable = 0;

    // Get the RDS status from the part.
	fmRdsStatus(1, 0);

	// Loop until all the RDS information has been read from the part.
	while(RdsFifoUsed)
    {
   		unsigned char bler[4];
        unsigned char field ;
        
        if(GrpLost)
		{
			RdsDataLost++;
		}

        // Gather the latest BLER info
        bler[0] = BleA;
        bler[1] = BleB;
        bler[2] = BleC;
        bler[3] = BleD;

        errorCount = 0;
        RdsGroups++;
        for (field = 0; field <= 3; field++)
        {
            if (bler[field] == UNCORRECTABLE)
            {
                errorCount++;
            }
            else
            {
                RdsValid[field]++;
            }
        }

	    if (errorCount < 4)
	    {
	        RdsBlocksValid += (4 - errorCount);
	    }

	    if (RdsIndicator)
	    {
			LED(1);
	    }
	    else
	    {
	    	LED(0);
	    }

	    RdsIndicator = 100; // Reset RdsIndicator

	    if (BleB <= rdsBlerMax[1])
	    {
	        group_type = BlockB >> 11;  // upper five bits are the group type and version

	    }
	    else
	    {
	        // Drop the data if more than two errors were corrected in block B
	        return;
	    }

	    switch (group_type) {
	        case RDS_TYPE_0A:

	        case RDS_TYPE_0B:

	            break;

	        case RDS_TYPE_10A:	//RDS_TYPE_2A:
	        {

	            break;
	        }

	        case RDS_TYPE_11A:	//RDS_TYPE_2B:
	        {

	            break;
	        }
	        case RDS_TYPE_4A:
	        {
	            // Clock Time and Date
	            break;
	        }
	        
	         case RDS_TYPE_3A:
	        {
	        	//Application Identification for Open Data
	        	break;
	        }
	        
	        case RDS_TYPE_12A:
	        {
	        	//Open Data Architecture
	        	break;
	        }
	        
	        case RDS_TYPE_124A:
	        {
	        	//Open Data Architecture
	        	
//	        	//Ver. 1.0
	        	PCHblocks[0] = (unsigned char)(BlockC >> 8);
				PCHblocks[1] = (unsigned char)(BlockC & 0xFF);
				PCHblocks[2] = (unsigned char)(BlockD >> 8);
				PCHblocks[3] = (unsigned char)(BlockD & 0xFF);
	            
	            u8 id = (unsigned char)((BlockC & 0x8000) >> 15 );
	            u16 counter = 0;
	         	char *Flash_ptr;
	         	u16 i = 0;
	         	u16 addr = 0xFF00;
	         	
	            Flash_ptr = (char *) addr + counter;
	            
	         	if (counter==0)
	         	{
	         		FCTL1 = FWKEY + ERASE;                    // Set Erase bit
					FCTL3 = FWKEY;                            // Clear Lock bit
					*Flash_ptr = 0;                           // Dummy write to erase Flash segment
					
					FCTL1 = FWKEY + WRT;                      // Set WRT bit for write operation
	         	}

				if(!((BleC == 0) & (BleD == 0)))
				{
					abflag=abflag;
				}
	            
	            for (i=0; i<4; i++)
				{
					*Flash_ptr++ = PCHblocks[i];                 // Write value to flash
					counter++;
					//wait_us(1);
				}
	            
	            if (counter==256)
	            {
	            	FCTL1 = FWKEY;                            // Clear WRT bit
  					FCTL3 = FWKEY + LOCK;                     // Set LOCK bit
  					
  					Flash_ptr = (char *) addr;
  					u8 array[256] = {0};
  					
  					for (i=0; i<=255; i++)
  					{
  						array[i]= *Flash_ptr++;
  					}
  					
  					counter = 0;
  					
	            }	            
	            
	            break;
	        }
	        
	        default:
	        {
	            break;
	        }
	    }
    
	    // Get the RDS status from the part.
		fmRdsStatus(1, 0);
    }
    
    // bpark pmo
    if (RdsFifoUsed == 0)
    {
    	LED(0);
    }
}

//-----------------------------------------------------------------------------
//LED - Anzeige -  wolf 
//sbit LED         = P1^5;  // Controls the LED on the baseboard
//-----------------------------------------------------------------------------
void LED(bool act)
{//P1.0 high = illuminated
	
		P1DIR |= 0x01;
		
		if (act == 1)
    	P1OUT |= 0x01;  //LED an
    else	  
    	P1OUT &= 0xFE;  //LED aus
    
}

//-----------------------------------------------------------------------------
// Helper function that sends the FM_TUNE_FREQ command to the part
//
// Inputs:
// 	frequency in 10kHz steps
//-----------------------------------------------------------------------------
void fmTuneFreq(int frequency)
{
	unsigned char i;
	
    // Put the ID for the command in the first byte.
    cmd[0] = FM_TUNE_FREQ; //0x20

	// Initialize the reserved section to 0
    cmd[1] = 0;

	// Put the frequency in the second and third bytes.
    cmd[2] = (unsigned char)(frequency >> 8);
	cmd[3] = (unsigned char)(frequency & 0x00FF);

	// Set the antenna calibration value.
    cmd[4] = (unsigned char)0;  // Auto

	 //unusual arguments must be 0x00.    
    for (i=5; i<8; i++ )
    {
    	cmd[i] = 0x00;
    }

    // Invoke the command
	si47xx_command(8, cmd, 0, rsp);
}

//-----------------------------------------------------------------------------
// Helper function that sends the FM_TUNE_STATUS command to the part
//
// Inputs:
// 	cancel: If non-zero the current seek will be cancelled.
//  intack: If non-zero the interrupt for STCINT will be cleared.
//
// Outputs:  // These are global variables and are set by this method
//  STC:    The seek/tune is complete
//  BLTF:   The seek reached the band limit or original start frequency
//  AFCRL:  The AFC is railed if this is non-zero
//  Valid:  The station is valid if this is non-zero
//  Freq:   The current frequency
//  RSSI:   The RSSI level read at tune.
//  ASNR:   The audio SNR level read at tune.
//  AntCap: The current level of the tuning capacitor.
//-----------------------------------------------------------------------------
char fmTuneStatus(unsigned char cancel, unsigned char intack)
{	
	
	unsigned char i;

    // Put the ID for the command in the first byte.
    cmd[0] = FM_TUNE_STATUS;

	// Put the flags if the bit was set for the input parameters.
	cmd[1] = 0x00;
    if(cancel) cmd[1] |= FM_TUNE_STATUS_IN_CANCEL;
	if(intack) cmd[1] |= FM_TUNE_STATUS_IN_INTACK;

	for (i=2; i<8; i++)
    {
    	cmd[i] = 0x00;
    }

    // Invoke the command
	si47xx_command(8, cmd, 16, rsp);

    // Parse the results
    STC    = !!(rsp[0] & STCINT);
    BLTF   = !!(rsp[1] & FM_TUNE_STATUS_OUT_BTLF);
    AFCRL  = !!(rsp[1] & FM_TUNE_STATUS_OUT_AFCRL);
    Valid  = !!(rsp[1] & FM_TUNE_STATUS_OUT_VALID);
    Freq   = ((int)rsp[2] << 8) | (int)rsp[3];
    RSSI   = rsp[4];
    ASNR   = rsp[5];
    AntCap = rsp[7];   
    
    return RSSI;
}

//-----------------------------------------------------------------------------
// Helper function that sends the FM_RDS_STATUS command to the part
//
// Inputs:
//  intack: If non-zero the interrupt for STCINT will be cleared.
//  mtfifo: If non-zero the fifo will be cleared.
//
// Outputs:
//  Status:      Contains bits about the status returned from the part.
//  RdsInts:     Contains bits about the interrupts that have fired related to RDS.
//  RdsSync:     If non-zero the RDS is currently synced.
//  GrpLost:     If non-zero some RDS groups were lost.
//  RdsFifoUsed: The amount of groups currently remaining in the RDS fifo.
//  BlockA:      Block A group data from the oldest FIFO entry.
//  BlockB:      Block B group data from the oldest FIFO entry.
//  BlockC:      Block C group data from the oldest FIFO entry.
//  BlockD:      Block D group data from the oldest FIFO entry.
//  BleA:        Block A corrected error information.
//  BleB:        Block B corrected error information.
//  BleC:        Block C corrected error information.
//  BleD:        Block D corrected error information.
//-----------------------------------------------------------------------------
void fmRdsStatus(unsigned char intack, unsigned char mtfifo)
{
    unsigned char i;
    
    // Put the ID for the command in the first byte.
    cmd[0] = FM_RDS_STATUS;	//0x24

	// Put the flags if the bit was set for the input parameters.
	cmd[1] = 0x00;
	if(intack) cmd[1] |= FM_RDS_STATUS_IN_INTACK;
    if(mtfifo) cmd[1] |= FM_RDS_STATUS_IN_MTFIFO;
	
	for (i=2; i<8; i++)
    {
    	cmd[i] = 0x00;
    }
	
    // Invoke the command
	si47xx_command(8, cmd, 16, rsp);

    // Parse the results
	Status      = rsp[0];
    RdsInts     = rsp[1];
	RdsSync     = !!(rsp[2] & FM_RDS_STATUS_OUT_SYNC);
    GrpLost     = !!(rsp[2] & FM_RDS_STATUS_OUT_GRPLOST);
    RdsFifoUsed = rsp[3];
    BlockA      = ((int)rsp[4] << 8) | (int)rsp[5];
    BlockB      = ((int)rsp[6] << 8) | (int)rsp[7];
    BlockC      = ((int)rsp[8] << 8) | (int)rsp[9];
    BlockD      = ((int)rsp[10] << 8) | (int)rsp[11];
    BleA        = (rsp[12] & FM_RDS_STATUS_OUT_BLEA) >> FM_RDS_STATUS_OUT_BLEA_SHFT;	//rsp[12] & C0 >> 6
    BleB        = (rsp[12] & FM_RDS_STATUS_OUT_BLEB) >> FM_RDS_STATUS_OUT_BLEB_SHFT;	//rsp[12] & 30 >> 4
    BleC        = (rsp[12] & FM_RDS_STATUS_OUT_BLEC) >> FM_RDS_STATUS_OUT_BLEC_SHFT;	//rsp[12] & 0C >> 2
    BleD        = (rsp[12] & FM_RDS_STATUS_OUT_BLED) >> FM_RDS_STATUS_OUT_BLED_SHFT;	//rsp[12] & 03 >> 0
}

//-----------------------------------------------------------------------------
// Set up general configuration properties:
//      Soft Mute Rate, Soft Mute Max Attenuation, Soft Mute SNR Threshold,
//      Blend Mono Threshold, Blend Stereo Threshold, Max Tune Error,
//      Seek Tune SNR Threshold, Seek Tune RSSI Threshold
//
// Note:
//     * RDS is only available on certain parts.  Please refer to the data
//       sheet for your part to determine if your part supports RDS.
//-----------------------------------------------------------------------------
static void si47xxFMRX_general_cfg(void)
{
    // Typically the settings used for stereo blend are determined by the 
    // designer and not exposed to the end user. They should be adjusted here.
    // If the user wishes to force mono set both of these values to 127.
    // si47xx_set_property(FM_BLEND_MONO_THRESHOLD, 30);
    // si47xx_set_property(FM_BLEND_STEREO_THRESHOLD, 49);

    // The softmute feature can be disabled, but it is normally left on.
    // The softmute feature is disabled by setting the attenuation property
    // to zero.
    //  si47xx_set_property(FM_SOFT_MUTE_RATE, 64);
    //  si47xx_set_property(FM_SOFT_MUTE_MAX_ATTENUATION, 16);
    //  si47xx_set_property(FM_SOFT_MUTE_SNR_THRESHOLD, 4);

    // The max tune error is normally left in its default state.  The designer
    // can change if desired.
    //  si47xx_set_property(FM_MAX_TUNE_ERROR, 30);
 
    // Typically the settings used for seek are determined by the designer
    // and not exposed to the end user. They should be adjusted here.
    si47xx_set_property(FM_SEEK_TUNE_SNR_THRESHOLD, 3);
    si47xx_set_property(FM_SEEK_TUNE_RSSI_THRESHOLD, 20);
}

//-----------------------------------------------------------------------------
// This function will set up some general items on the hardware like
// initializing the RDS and STC interrupts.
//
// Note:
//     * RDS is only available on certain parts.  Please refer to the data
//       sheet for your part to determine if your part supports RDS.
//-----------------------------------------------------------------------------
static void si47xxFMRX_hardware_cfg(void)
{
	// Enable the RDS and STC interrupt here
    si47xx_set_property(GPO_IEN, GPO_IEN_STCIEN_MASK | GPO_IEN_RDSIEN_MASK);
}

void SCL_low(void)                     // pin 48 (SCL) auf low setzen
{
	IICO &= SCLL;
	wait_us(1);                         // 1ms warten (rise time, fall time usw.)
}

void SCL_high(void)                    // pin 48 (SCL) auf high setzen
{
	IICO |= SCLH;
}

void SCL_clock(void)
{
	SCL_high();
	SCL_low();
}

void SDA_low(void)                     // pin 47 (SDA) auf low setzen
{
	SDA_write();	
	IICO &= SDAL;
}

void SDA_high(void)                    // pin 47 (SDA) auf high setzen
{
	SDA_write();
	IICO |= SDAH;
}

void SDA_write(void)                   // pin 47 (SDA) auf output setzen
{
  IICD |= SDAH;
}

void SDA_read(void)                    // pin 47 (SDA) auf input setzen
{
  IICD &= SDAL;
}

void SDA_OUT(bool Data)
{
  SDA_write();
  if (Data == 0) 
    P3OUT &= 0xFB;
  else
    P3OUT |= 0x04;

}

bool SDA_IN()
{
  SDA_read();
  if (P3IN & 0x04)   //testen
    return 1;
  else
    return 0;
}

void SCL_write(void)                   // pin 48 (SCL) auf output setzen
{
  IICD |= SCLH;
}

void SCL_read(void)                    // pin 48 (SCL) auf input setzen
{
  IICD &= SCLL;
}

void SEN_low()
//SEN  P4.0  TP7
{
	P4SEL &= 0xFE;
	P4REN |= 0x01;
	P4DIR |= 0x01;
	P4OUT &= 0xFE; 
	P4OUT |= 0x01; 
	P4OUT &= 0xFE;
}

void SEN_high()
//SEN  P4.0  TP7
{
	P4SEL &= 0xFE;
	P4REN |= 0x01;
	P4DIR |= 0x01;
	P4OUT &= 0xFE;
	P4OUT |= 0x01;
}
