//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <stddef.h>
#include <stdbool.h>
#include "typedefs.h"
#include <msp430f2274.h>
#include "si47xxFMRX.h"
#include "FMRXtest.h"
#include "propertydefs.h"

//-----------------------------------------------------------------------------
// Defines
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Function prototypes
//-----------------------------------------------------------------------------
void _nop_(void);

//-----------------------------------------------------------------------------
// Externals
//-----------------------------------------------------------------------------
extern u8 RdsAvailable;

//-----------------------------------------------------------------------------
// Tune to a station known to transmit RDS data. Wait a few seconds and
// then stop execution so you can examine the various RDS buffers.  If you
// place a break point on the si47_rdsGetBler, you'll have to wait for the
// second time the code stops there.
//-----------------------------------------------------------------------------
void test_FMRXrds(void)
{
    u16 bler;
    u8 updatecntr=0;
    u16 rdsConfig=0;
    
    //si47xxFMRX_initialize();
    si47xxFMRX_set_volume(63); // full volume, turn off mute

	// Setup the RDS Interrupt to interrupt when RDS data is available.
	si47xx_set_property(FM_RDS_INTERRUPT_SOURCE, FM_RDS_INTERRUPT_SOURCE_RECV_MASK);
	si47xx_set_property(FM_RDS_INTERRUPT_FIFO_COUNT, 1);
	//si47xx_SPIset_property(FM_RDS_INTERRUPT_SOURCE, FM_RDS_INTERRUPT_SOURCE_RECV_MASK);
	//si47xx_SPIset_property(FM_RDS_INTERRUPT_FIFO_COUNT, 1);

	// Setup the RDS configuration to enable RDS and allow any error condition.
	rdsConfig |= FM_RDS_CONFIG_RDSEN_MASK;
	rdsConfig |= FM_RDS_BLETH_UNCORRECTABLE << FM_RDS_CONFIG_BLETHA_SHFT;
	rdsConfig |= FM_RDS_BLETH_UNCORRECTABLE << FM_RDS_CONFIG_BLETHB_SHFT;
	rdsConfig |= FM_RDS_BLETH_UNCORRECTABLE << FM_RDS_CONFIG_BLETHC_SHFT;
	rdsConfig |= FM_RDS_BLETH_UNCORRECTABLE << FM_RDS_CONFIG_BLETHD_SHFT;
    si47xx_set_property(FM_RDS_CONFIG, rdsConfig);
    //si47xx_SPIset_property(FM_RDS_CONFIG, rdsConfig);
	
	si47xx_set_property(FM_ANTENNA_INPUT, 0);
	
	//RdsTestInProc = 1;
	si47xxFMRX_tune(10060);    // tune to a station that transmits RDS
	
	//si47xx_GET_AGC_Status();

    	//si47xxFMRX_tune(9580);    // tune to a station that transmits RDS
		u8 locRSSI = fmTuneStatus(0,0);
		//RSSI_LED(locRSSI);
	
	LED(1);
	wait_ms(1000);
	LED(0);
	wait_ms(1000);
	//wait_ms((60-locRSSI)*10);
	LED(1);
	wait_ms(1000);
	LED(0);

    initRdsVars();             // Initialize RDS variables
    
	
    // Allow this loop to run for several seconds before stopping execution.
    // The RDS data can be examined in the following variables:
    // piDisplay, ptyDisplay, rtDisplay, rtSimple, psDisplay,
    // ctDayHigh, ctDayLow, ctHour, ctMinute, ctOffset
    RdsAvailable=true;
    while(1) 
    {
		if(RdsAvailable)
		{
	        //locRSSI=fmTuneStatus(0,0);
	        //RSSI_Led(locRSSI);
	        updateRds();
	        if (!updatecntr++) {
	            // Update bler every time counter rolls over
	            si47_rdsGetBler(&bler);
	        }
		}
    }
}

//-----------------------------------------------------------------------------
// Test the power down function of the part.  Unlike the Si470x parts the Si47xx
// parts need to be fully initialized after power down.
//-----------------------------------------------------------------------------
void test_FMRXpowerCycle(void)
{
    si47xxFMRX_powerdown(); // Test powerdown with audio hi-z
    wait_ms(2000);    // Wait 5s before starting next test
    si47xxFMRX_powerup();   // Powerup without reset, the part will need to be tuned again.
    si47xxFMRX_configure();
    si47xxFMRX_set_volume(63);     // full volume, turn off mute
	si47xxFMRX_tune(10230);
	wait_ms(2000);    // Wait 5s before starting next test
}
