//******************************************
//* ST7528
//LCD-Controller
//******************************************
#include "typedefs.h"
#include "hardware.h"
#include "string.h"
#include "Arial_8.h"
#include "Verdana16.h"
#include "DejaVu_10.h"
#include "stdlib.h"
#include "fet140_flashwrite_01.h"
#include <msp430F2274.h>
#include <stdbool.h>

//LCD-Befehls-Konstanten
#define DisplayOn			0xAF 
#define DisplayOff			0xAE
#define DisplayReversOff	0xA6	
#define DisplayReversOn		0xA7

//******************************************
//Function Prototypes
void LCDWriteLow(u8 PA, u8 col, char* Text, u8 Font, u8 inv, bool del);
void DrawBMP(unsigned char * bmpd);

const u16 cDRAMu = 0x0D700;
const u16 cDRAMo = 0x0D900;

void lcd_write_byte ( unsigned char byte_i )
{
	unsigned char i;
	
	for ( i = 0; i < 8; i++ )
	{
		Clock_L();
		 
		if ( ( (byte_i << i) & 0x80 ) == 0x80 )
		 	P1OUT |= data; 	//sid = 1;
		else
			P1OUT &= ~data;		//sid = 0;
		
		Clock_H();
	}
}

void lcd_write_command ( unsigned char command)
{
	CS_L();
	lcd_write_byte(command);
	CS_H();
}

void lcd_write_one_data ( unsigned char data_i )
{ //4*  = schwarz
	CS_L();
	lcd_write_byte ( 0xe8 );        //set DDC
	lcd_write_byte ( 3 );           //set No. of data bytes 1
	lcd_write_byte ( data_i );
	lcd_write_byte ( data_i );
	lcd_write_byte ( data_i );
	lcd_write_byte ( data_i );
	CS_H();
}

void lcd_write_data ( unsigned char gray_level )
{
	CS_L();
	lcd_write_byte ( 0xe8 );        //set DDC
	lcd_write_byte ( 3 );           //set No. of data bytes 1
	CS_H();
}

void lcd_write_Xdata(int counter, unsigned char datas)
{//counter=Spalten
	int i;
	
	CS_L();
	lcd_write_byte ( 0xe8 );        //set DDC
	lcd_write_byte ( (counter*4)-1);           //set No. of data bytes 1
	for (i=0; i<counter; i++)
	{
		lcd_write_byte (datas);
		lcd_write_byte (datas);
		lcd_write_byte (datas);
		lcd_write_byte (datas);
	}
	CS_H();
}

void set_page ( unsigned char page )
{
	lcd_write_command ( 0xb0 | ( page+2)%4 );
	lcd_write_command ( 0xb0 | ( page+2)%4 );     // Page must be set twice !!!
}


void set_column ( unsigned char column )
{
	lcd_write_command ( 0x10 | ((column >> 4)&0x0f) );     //MSB
	lcd_write_command ( 0x00 | (column & 0x0f) );   //LSB
}

void ClearLCDold()
{
	int i;
	int j;
	
		for (i=0;i<4;i++)
		{
			set_page(i);
	
			for (j=0;j<160;j++)
			{
				set_column(j);
				lcd_write_one_data(0x00); 
			}
		}	
}

void ClearLCD()
{
	u8 j;
	
	for (j=0;j<4;j++)
	{
	  set_page(j);
	  set_column(0);

      lcd_write_Xdata(64, 00);      // 160 * 4 / 256 = 2,5 
      lcd_write_Xdata(64, 00);       
      lcd_write_Xdata(32, 00);      
	}
}


void set_gray_scale(void)
{
	unsigned char i;
	//EXT = 1
	lcd_write_command (0x38);     //Mode Set, 2-byte instruction
	lcd_write_command (0x05);     //frame 77Hz, BE level 2, EXT = 1
	
	lcd_write_command (0x80);     // 1st frame
	lcd_write_command (0);
	lcd_write_command (0x81);     // 2nd frame
	lcd_write_command (0);
	lcd_write_command (0x82);     // 3rd frame
	lcd_write_command (0);
	lcd_write_command (0x83);     // 4th frame
	lcd_write_command (0);
	
	//set gray scale, 45PWM: 0~0x2d ( 0~45 )
	for(i=1;i<15;i++)
	{
		lcd_write_command (0x80+i*4);     // 1st frame
		lcd_write_command (30+i);
		lcd_write_command (0x81+i*4);     // 2nd frame
		lcd_write_command (30+i);
		lcd_write_command (0x82+i*4);     // 3rd frame
		lcd_write_command (30+i);
		lcd_write_command (0x83+i*4);     // 4th frame
		lcd_write_command (30+i);
		
		lcd_write_command (0xbc);     // 1st frame
		lcd_write_command (45);
		lcd_write_command (0xbd);     // 2nd frame
		lcd_write_command (45);
		lcd_write_command (0xbe);     // 3rd frame
		lcd_write_command (45);
		lcd_write_command (0xbf);     // 4th frame
		lcd_write_command (45);
	}

	//EXT = 0
	lcd_write_command(0x38);     //Mode Set, 2-byte instruction
	lcd_write_command(0x04);     //frame 77Hz, BE level 2, EXT = 0
}

void lcd_initial(void)
{	
	//u8 i;
	
	Reset_LH();
	Reset_LH();
	
	set_gray_scale();
	
	//user application setup by internal instructions
	lcd_write_command (0x48);     //display duty select
	lcd_write_command (40);     // 1/32 duty
	
	lcd_write_command (0xa1);     //ADC select S0->S159, 0xa1: S159->S0
	
	lcd_write_command (0xc8);     //SHL select C0->C99, 0xc4: C99->C0
	
	lcd_write_command (0x40);     //Initial display line register
	lcd_write_command (0);
	
	lcd_write_command (0x44);     //COM0 register select
	lcd_write_command (34);     //war mal
	
	//user LCD power setup by internal instructions
	//V0 = ( 1 + Rb/Ra ) * ( 1 - (63 - a)/210 ) * VREF, VREF = 2.1V when Ta = 25
	lcd_write_command (0xab);     //Oscillator ON
	
	lcd_write_command (0x24);     //( 1 + Rb/Ra ) = 5.1, regulator resistor
	lcd_write_command (0x81);     //Electronic Volume Resistor Select
	lcd_write_command (0x16);     // a = 24
	
	lcd_write_command (0x52);     // 1/7 bias, LCD Bias register select
	lcd_write_command (0x65);     //Vout = 4*VDD, DC-DC step-up register select
	wait_ms(500);
	lcd_write_command (0x2f);     //Power Control VC,VR,VF
	
	lcd_write_command (0xaf);     //display on, 0xae -> off
	
	lcd_write_command ( 0xa5 );     //entire on
	wait_ms(500);
	lcd_write_command ( 0xa4 );     //entire on off
	
	//EXT = 0
	lcd_write_command ( 0x38 );     //Mode Set, 2-byte instruction
	lcd_write_command ( 0x04 );     //frame 77Hz, BE level 2, EXT = 0
	
	ClearLCD();
}

//*************************************************
//String wird bergeben
//und an aktueller CurPos ausgegeben
void LCDWrite(char* Text, u8 pa, u8 sp, u8 Font, u8 inv, bool del)
{
	//i = strlen(Text);
	
	LCDWriteLow(pa, sp, Text, Font, inv, del);
}

//*************************************************
//3-Line-SPI MPU interface
//And the DDRAM column address pointer will be increased by one automatically. - n
//Font 1= 8pixel     2=16pixel
void LCDWriteLow(u8 PA, u8 col, char* Text, u8 Font, u8 inv, bool del)
{
		u8 i;
		u8 j;
		u8 k;
		u8 m;
		int cp;	//char-pointer
		u8 bm;	//bit-muster
		u8 len;
		u8 sp=col;
		int BitIndex; 
		int ByteIndex;
		u8 BitShift;
		int PixelBreiten = 0;
		char *Flash_ptr_o;
		char *Flash_ptr_u;
				
		len = strlen(Text);		
				
		set_page(PA);
		set_column(col);	
				
		CS_L();
									
		switch (Font)
		{
			//***************************************************************************************
			case 1: //8pixel Schrift
					//Text Convertieren in Bitfolge	aus Hagen-Headerdatei	
					for (i=0; i<len; i++) //Text-Zeichen-Loop
					{
						//DataOut(0xE8);									//Data Direction Command (switch von Default CommandMode auf DataMode	
						//k = GetCharWidth(Text[i], Font);
	
						cp = Text[i];

						for (j=0; j< k ; j++) //Muster-Loop pro Zeichen
						{
							bm = Arial_8[cp];	//Verdana16[cp];  //DejaVu_16
							if (inv) bm=~bm;
							
							set_column(sp++);					
							lcd_write_one_data(bm);
							cp++;
						}
						
						set_column(sp++);
						if (inv)
							lcd_write_one_data(0xFF);	//LeerSpalte nach jedem Buchstaben ausgeben
						else
							lcd_write_one_data(0x00);	//LeerSpalte nach jedem Buchstaben ausgeben
					
					}
					
				break;
			
			//***************************************************************************************
			case 2:	//16pixel Schrift			//Text Convertieren in Bitfolge	aus Hagen-Headerdatei	
					
					for (i=0; i<len; i++) //Text-Zeichen-Loop
					{
						//DataOut(0xE8);									//Data Direction Command (switch von Default CommandMode auf DataMode	
						//k = GetCharWidth(Text[i], Font);
	
						cp = Text[i];

						for (j=0; j< k ; j++) //Muster-Loop pro Zeichen
						{
							bm = Verdana16[cp];	//Verdana16[cp];  //DejaVu_16
							if (inv) bm=~bm;
							
							set_page(PA);
							set_column(sp);
							
							lcd_write_one_data(bm);
							cp++;
							
							bm = Verdana16[cp];
							if (inv) bm=~bm;
							set_page(PA+1); //2te Zeile
							set_column(sp);
							lcd_write_one_data(bm);
							cp++;
							sp++;
						}
						
						set_page(PA);
						set_column(sp);
						
						if (inv)
							lcd_write_one_data(0xFF);	//LeerSpalte nach jedem Buchstaben ausgeben
						else
						  lcd_write_one_data(0x00);	//LeerSpalte nach jedem Buchstaben ausgeben
						  						  
						  
						set_page(PA+1); //2te Zeile
						set_column(sp);
						
						if (inv)
							lcd_write_one_data(0xFF);	//LeerSpalte nach jedem Buchstaben ausgeben
						else
						  lcd_write_one_data(0x00);	//LeerSpalte nach jedem Buchstaben ausgeben

						
						sp++;
					}
					
					break;			
									
				//***************************************************************************************
				case 3: //10pixel MONO  3 (0..2)Zeilen mit Unterlngen
							//1. 5*8bit (oben)   dann 5*2bit Rest	(unten)				
							//braucht 320 Byte RAM
														
					BCSCTL1 = CALBC1_1MHZ; // Set DCO to xxMHz
					DCOCTL = CALDCO_1MHZ;
					FCTL2 = FWKEY + FSSEL0 + FN0;             // MCLK/2 for Flash Timing Generator
					wait_ms(1);
					
					Flash_ptr_u = (char *) cDRAMu + sp;				// Initialize Flash pointer
					Flash_ptr_o = (char *) cDRAMo + sp;				// Initialize Flash pointer
		
					switch (PA)
					{
						case 0:	//Page 0+1
							
							if(del)
							{
								make_IFlash_write1();
								wait_ms(1);
								*Flash_ptr_o = 0;								// Dummy write to erase Flash segment
								wait_ms(1);
							}
							make_IFlash_write2();
							
							set_page(PA);
							
							for (i=0; i<len; i++) //Text-Zeichen-Loop
							{		
								ByteIndex = Text[i] - 0x1F -1;  //Zeichen - FirstChar
								ByteIndex = ByteIndex * 10;
								ByteIndex +=  8;		//Header Offset 
																						
								for (m=0; m<5; m++)	//breite
								{
									set_page(PA);
									set_column(sp);
								  	bm = DejaVu_Mono_10[ByteIndex];
								  	lcd_write_one_data(bm);
								  
									//2bit-Rest
									set_page(PA+1);
									set_column(sp);
									bm = DejaVu_Mono_10[ByteIndex+5];
									
									//DRAMo[sp] = bm;
									*Flash_ptr_o = bm;						// Write value to flash
									
									lcd_write_one_data(bm);
																	  
									ByteIndex++;
									sp++;
									*Flash_ptr_u++;
									//*Flash_ptr_u2++;
								  	*Flash_ptr_o++;
								}
									
								lcd_write_one_data(0x00);	//LeerSpalte nach jedem Buchstaben ausgeben
							}
					 		
					 		while(sp<160)
							{
								*Flash_ptr_o = 0x00;
								
								sp++;
							  	*Flash_ptr_o++;
							}
								
							break;
					 		
					 	case 1: // Page 1+2
					 									 	
				 			if(del)
							{
								make_IFlash_write1();
								wait_ms(1);
								*Flash_ptr_u = 0;								// Dummy write to erase Flash segment
								wait_ms(1);
							}
							make_IFlash_write2();
							
							set_page(PA);
				
							for (i=0; i<len; i++) //Text-Zeichen-Loop
							{		
								ByteIndex = Text[i] - 0x1F -1;  //Zeichen - FirstChar
								ByteIndex = ByteIndex * 10;
								ByteIndex +=  8;		//Header Offset 
																						
								for (m=0; m<5; m++)	//breite
								{
									set_page(PA);
									set_column(sp);
								  	bm = DejaVu_Mono_10[ByteIndex]<<3; //nach unten
								  	//DRAMo[sp] = DRAMo[sp]| bm;
								 	
								 	//bm = DRAMo[sp] | bm;
									bm = *Flash_ptr_o | bm;

								  	lcd_write_one_data(bm);
								  
								  	//Page 2 ErstRest 3pix 
								  	set_page(PA+1);
								  	set_column(sp);
								  	bm = DejaVu_Mono_10[ByteIndex]>>5;
								  	
								  	//DRAMu[sp] = bm;
									u8 bmTmp = bm;						// Write value to flash
									
								  	//lcd_write_one_data(bm);
								  												  
								  	//rest 2pix
								  	set_column(sp);
								  	bm = DejaVu_Mono_10[ByteIndex+5]<<3;
//								  	bm = DRAMu[sp] | bm;
								  	//DRAMu[sp] = DRAMu[sp] | bm;

									bm = bmTmp | bm;
									*Flash_ptr_u = bmTmp | bm;
								  	
								  	lcd_write_one_data(bm);
								  								  
								  	ByteIndex++;
								  	sp++;
								  	*Flash_ptr_u++;
								  	*Flash_ptr_o++;
								}
								
								lcd_write_one_data(0x00);	//LeerSpalte nach jedem Buchstaben ausgeben
							}						 	
					 		
					 		while(sp<160)
							{
								*Flash_ptr_u = 0x00;
								
								sp++;
								*Flash_ptr_u++;
							}
						
							break;

					 	case 2:	// Page 2+3
					 			//2pix oben,  8pix unten
					 			//5pix aus dem Speicher DRAM holen
				 		    
				 		    set_page(PA);
								
							for (i=0; i<len; i++) //Text-Zeichen-Loop
							{		
								ByteIndex = Text[i] - 0x1F -1;  //Zeichen - FirstChar
								ByteIndex = ByteIndex * 10;
								ByteIndex +=  8;		//Header Offset 
																						
								for (m=0; m<5; m++)	//breite
								{
									set_page(PA);
									set_column(sp);
									bm = DejaVu_Mono_10[ByteIndex]<<6;
									
									//bm = DRAMu[sp] | bm; 
									bm = *Flash_ptr_u | bm;
									
									lcd_write_one_data(bm);
									  
									// Page 3 6pix und Rest 2pix 
									set_page(PA+1);
									set_column(sp);
									bm = DejaVu_Mono_10[ByteIndex]>>2;
									bm = bm | DejaVu_Mono_10[ByteIndex+5]<<6; //unten
									lcd_write_one_data(bm);
									  								  
									ByteIndex++;
									sp++;
									*Flash_ptr_u++;
							  
								}
							
								lcd_write_one_data(0x00);	//LeerSpalte nach jedem Buchstaben ausgeben
							}			
				 									
						break;
					}
							 
					stop_IFlash_write();
					wait_ms(1);
		
					BCSCTL1 = CALBC1_12MHZ; // Set DCO to xxMHz
					DCOCTL = CALDCO_12MHZ;
					wait_ms(1);
					
					break;
				
				//***************************************************************************************
				case 4: //20pixel MONO  1 Zeile  
				
					BCSCTL1 = CALBC1_1MHZ; // Set DCO to xxMHz
					DCOCTL = CALDCO_1MHZ;
					FCTL2 = FWKEY + FSSEL0 + FN0;             // MCLK/2 for Flash Timing Generator
				
					Flash_ptr_u = (char *) cDRAMu + sp;				// Initialize Flash pointer

					make_IFlash_write1();
					*Flash_ptr_u = 0;								// Dummy write to erase Flash segment
					wait_us(1);
					make_IFlash_write2();
					
					//---------------------------------------
							
					set_page(PA);
					
					for (i=0; i<len; i++) //Text-Zeichen-Loop
					{		
						ByteIndex = Text[i] - 0x1F -1;  //Zeichen - FirstChar
						ByteIndex = ByteIndex * 30;
						ByteIndex +=  8;		//Header Offset 
																				
						for (m=0; m<10; m++)	//Breite
						{
							set_page(PA);
							set_column(sp);
							bm = DejaVu_Mono_20[ByteIndex];
							lcd_write_one_data(bm);
						  
							set_page(PA+1);
							set_column(sp);
							bm = DejaVu_Mono_20[ByteIndex+10];
							lcd_write_one_data(bm);
	
							set_page(PA+2);
							set_column(sp);
							bm = DejaVu_Mono_20[ByteIndex+20];
//							DRAMu[sp] = bm;

							*Flash_ptr_u = bm; //ntig?
							
							lcd_write_one_data(bm);
						  								  
						  	ByteIndex++;
						  
							sp++;
							*Flash_ptr_u++;
						}
						
						lcd_write_one_data(0x00);	//LeerSpalte nach jedem Buchstaben ausgeben
					}
								
					while(sp<160)
					{
						*Flash_ptr_u = 0x00;
						
						sp++;
						*Flash_ptr_u++;
					}
							
					stop_IFlash_write();
		
					BCSCTL1 = CALBC1_12MHZ; // Set DCO to xxMHz
					DCOCTL = CALDCO_12MHZ;
					wait_ms(1);

					break;
					
					//***************************************************************************************
					case 5: //TEST   10pixel MONO  4 Zeilen ohne Unterlngen (g,j,q,p)
							//1. 5*8bit dann 5*2bit Rest					
							
						set_page(PA);
						for (i=0; i<len; i++) //Text-Zeichen-Loop
						{		
							ByteIndex = Text[i] - 0x1F -1;  //Zeichen - FirstChar
							ByteIndex = ByteIndex * 10;
							ByteIndex +=  8;		//Header Offset 
																					
							for (m=0; m<5; m++)
							{
								bm = DejaVu_Mono_10[ByteIndex];
							 	set_column(sp);
							  	lcd_write_one_data(bm);
							  	ByteIndex++;
							  	sp++;
							}
							
							lcd_write_one_data(0x00);	//LeerSpalte nach jedem Buchstaben ausgeben
						}
						 
					break;
		}
		
		CS_H();
}

//*************************************************
void LCD_Command(u8 Data)
//wird default vom LCD eingestellt
{}

//*************************************************
//gibt zum Zeichensatz die OffsetPosition zurck
//FontName z.B. 'Font_DejaVu_16_vert'
int GetOffset(char* FontName)
{
	int charpointer;
  	
	if (FontName == "Font_DejaVu_16_vert") 
	 	charpointer = 16;
	 	
	if (FontName == "Font_DejaVu_8_vert") 
	 	charpointer = 16;
 			
	return(charpointer);
}

void show_display(unsigned char *lcd_string)
{
 unsigned char page;
 unsigned char col;
 
 for (page=0xB0;page<0xC0;page++)		/*write to page 0 then go to mext page .*/ 
 {										/*     128pixels / 8per page = 16 pages    */
	  	DataOut(page);					/*Set page address*/
	  	DataOut(0x10);					/*Set column address MSB*/
	  	DataOut(0x00);					/*Set column address LSB*/
		for(col=0;col<128;col++)				/*each page has 128 pixel columns*/
	  	{
		   DataOut(*lcd_string);				/*16 level grayscale; write each byte 4 times*/
		   DataOut(*lcd_string);
		   DataOut(*lcd_string);
		   DataOut(*lcd_string++);			/*increment to next byte of data*/
		}
	}
}

void lcd_test()
{

	lcd_write_command(DisplayOff);
	lcd_write_command(DisplayOff);	
	LCDWrite("Start Demo-Sequenz 4.5", 0, 0, 2,0,true);
	lcd_write_command(DisplayOn);
	lcd_write_command(DisplayOn);
	wait_ms(4000);
	ClearLCD();

	//10er
	LCDWrite("Atomausstieg gjq ", 0, 0, 3, 0,true);
	LCDWrite("gRangliste mit Snder -gjq", 1, 0, 3, 0,true);
	LCDWrite("gDie perfekte Hochzeit -gji ", 2, 0, 3, 0,false);
	wait_ms(4000);
	
	ClearLCD();		
	LCDWrite("Die Hochzeit", 0, 0, 4, 0,true);
	LCDWrite("Rebellen starten Offensive", 2, 0, 3, 0,true);
	lcd_write_command(DisplayOn);
	lcd_write_command(DisplayOn);
	wait_ms(4000);
	
	lcd_write_command(DisplayOff);
	lcd_write_command(DisplayOff);
	ClearLCD();		
	LCDWrite("Wenn Herz Hirn besiegt", 0, 0, 1, 0,true);
	LCDWrite("Amerikaner sehen schwarz", 1, 0, 1, 0,false);
	LCDWrite("Die perfekte Hochzeit", 2, 0, 1, 0,false);
	LCDWrite("Atomausstieg jetzt ", 3, 0, 1, 0,false);
	
	lcd_write_command(DisplayOn);
	lcd_write_command(DisplayOn);
	wait_ms(4000);
	lcd_write_command(DisplayOff);
		
	ClearLCD();		
	//DrawBMP(Picture); //gs
	lcd_write_command(DisplayOn);		  	
}

void DrawBMP(unsigned char * bmpd)
{
	u8 sp;
	u8 p;
	u8 p2;	//Korr.
	int pt=0;
	u8 i;
	
	i = DisplayOff;
	i = 0xA4;
	lcd_write_command(i); 

	for (p=0; p<4; p++)

	{
		for (sp=0; sp<160; sp++)	
		{	
			p2=p;
			if (p==1) p2=2;
			if (p==2) p2=1;			
			set_page(p2);
	    	set_column(sp);
			lcd_write_one_data(bmpd[pt]);
			pt++;
		}
	}
	lcd_write_command(DisplayOn);
	
	lcd_write_command(DisplayReversOn);
	wait_ms(20);
	lcd_write_command(DisplayReversOff);
}

void EPZ(u8 p)
{ //ein paar Zeichen an der Page p ausgeben
	u8 j;
	u8 pt=240;
	set_page(p);
	for (j=0;j<25;j++)
	{
		set_column(j);
		lcd_write_one_data(Arial_8[pt]);
		pt++;
	}
}

void putbmp_ST7528(unsigned char * bmpd) 
{
unsigned int  num;
unsigned char numx, numy;	/* help variable, temp counter  	     */
unsigned char  * hv; //flash
   /* read number of display bytes Bugfix Windows Paintbrush doesn't 
      calculate picture size in byte all the time 			     */
	num = (unsigned int) ( *(bmpd+0x03)*256 + *(bmpd+0x02));  /* Bugfix       */

	for ( numx=0; numx < 4; numx++ )
	{	     	       /* display hole buffer               */
		set_page ( numx  );
		hv = bmpd + num - numx -1;  	       /* calculate display data start address */
		for ( numy = 0; numy < 160; numy++) 
		{	
			set_column ( numy );                   
			lcd_write_one_data( ~(*hv) ); 	       // write data byte as  
			hv -= 4;		               /* decrement picture data array pointer      */
		}
	}
}
