//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <stddef.h>
//#include "typedefs.h"
#include <msp430f2274.h>
//#include "i2c.h"
#include <stdbool.h>
#include "flash.h"

//-----------------------------------------------------------------------------
// Defines
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Function prototypes
//-----------------------------------------------------------------------------
void ioSPI_write_F(u8 number_bytes, u8 *data_out);

//-----------------------------------------------------------------------------
// Externals
//-----------------------------------------------------------------------------

// Inputs:
//       data_out: The data to write
//-----------------------------------------------------------------------------
void ioSPI_rawDataOut_F(unsigned char data_out)
{ //byte wird bitweise serialisiert
    int i;
    for (i=7; i>=0; i--)
    {
        SCL_low_F();	//SCLK = 0;
        SDA_OUT_F((data_out >> i) & 0x01); //SDIO = ((data_out >> i) & 0x01);
        wait_us(25);   // tLow
        SCL_high_F();	//SCLK = 1;
    }
}

//-----------------------------------------------------------------------------
// This routine reads one byte of data from the 3 wire interface
//
// Inputs:
//       data_in: The variable containing the data to read
//-----------------------------------------------------------------------------
void ioSPI_rawDataIn_F(unsigned char *data_in) //Idata
{
    int i;
		
    for (i=0; i<8; i++)
    {
        SCL_low_F(); //SCLK = 0;

        *data_in = ((*data_in << 1) | SDA_IN_F());
        
        SCL_high_F(); //SCLK = 1;
    }
}

//-----------------------------------------------------------------------------
// This routine reads an array of bytes from the si47xx part
//
// Inputs:
//		number_bytes: Number of bytes to read
//		data_in:	Destination array for data read
//-----------------------------------------------------------------------------
void ioSPI_read_F(u8 command[4], u8 number_bytes, u8 *data_in)
{
	u8 tmp[16]; //idata
	int i;
	
	ioSPI_write_F(1, command);
		
	for (i=0; i<number_bytes; i++)
	{
		ioSPI_rawDataIn_F(tmp); //*data_in
		data_in[i] = *tmp; //*data_in
	}
}

void ioSPI_memory_read_F(u8 command[4], u16 number_bytes, u8 *data_in)
{
	u8 tmp[140]; //idata
	//u8 cmdBytes;
	u16 i;
	
	//cmd[0]=command;
	ioSPI_write_F(4, command);
		
	for (i=0; i<number_bytes; i++)
	{
		ioSPI_rawDataIn_F(tmp); //*data_in
		data_in[i] = *tmp; //*data_in
	}
}

//-----------------------------------------------------------------------------
// Inputs:
//		number_bytes: Number of bytes to write
//		data_out:	  Source array for data to be written
//-----------------------------------------------------------------------------
void ioSPI_write_F(u8 number_bytes, u8 *data_out)
{
	u8 i;
		
	for (i=0; i<number_bytes; i++)
	{
		ioSPI_rawDataOut_F(data_out[i]);
	}
}

void Write_Enable(void)
{
	u8 cmd[5];
	
	cmd[0] = 0x06;	//write enable
	SEN_low_F();
	ioSPI_write_F(1,cmd);
	SEN_high_F();
}

void Chip_Erase(void)
{
	u8 cmd[5];
	
	Write_Enable();
	
	cmd[0] = 0x60;	//chip erase
	SEN_low_F();
	ioSPI_write_F(1,cmd);
	SEN_high_F();
}

void Block_Erase(u8 Block)
{
	u8 cmd[5];
	u8 i;
	u32 addr;
	
	addr = 0x10000 * Block;
	
	Write_Enable();
	
	cmd[0] = 0x52;	//block erase
	for(i=1;i<=3;i++) //addr.
	{
		cmd[i] =(unsigned char)((addr >> (3-i)*8) & 0xFF );
	}
	SEN_low_F();
	ioSPI_write_F(4,cmd);
	SEN_high_F();
}

void Sector_Erase(u8 id)
{
	u8 cmd[5];
	u8 i;
	u32 addr;
	
	addr = 0x1000 * id;
	
	Write_Enable();
	
	cmd[0] = 0x20;	//sector erase
	for(i=1;i<=3;i++) //addr.
	{
		cmd[i] =(unsigned char)((addr >> (3-i)*8) & 0xFF );
	}
	SEN_low_F();
	ioSPI_write_F(4,cmd);
	SEN_high_F();
}

void Write_Byte(u8 id, u8 Byte)
{
	u8 cmd[5];
	u8 i;
	u32 addr;
	
	addr = 0x1000 * id;
	Write_Enable();
	
	cmd[0] = 0x02;	//write Byte
	for(i=1;i<=3;i++) //addr.
	{
		cmd[i] =(unsigned char)((addr >> (3-i)*8) & 0xFF );
	}
	cmd[4] = Byte;	//Data
	SEN_low_F();
	ioSPI_write_F(5,cmd);
	SEN_high_F();
}

void Start_AI(u8 id, u8 Byte)
{
	u8 cmd[5];
	u8 i;
	u32 addr;
	
	addr = 0x1000 * id;
	
	Write_Enable();
	
	cmd[0] = 0xAF;	//AI
	for(i=1;i<=3;i++) //addr.
	{
		cmd[i] =(unsigned char)((addr >> (3-i)*8) & 0xFF );
	}
	cmd[4] = Byte;
	SEN_low_F();
	ioSPI_write_F(5,cmd);
	SEN_high_F();
}

void Write_Byte_AI(u8 Byte)
{
	u8 cmd[5];
	
	cmd[0] = 0xAF;	//AI
	cmd[1] = Byte;
	
	SEN_low_F();
	ioSPI_write_F(2,cmd);
	SEN_high_F();
}

void Read_Byte_F(u8 group, u8 retBytes, u8 *data_in)
{
	u8 cmd[5];
	u8 ret[140];
	u8 i;
	u32 addr;
	
	addr = 0x1000 * group;
	
	cmd[0] = 0x03;	//read
	for(i=1;i<=3;i++) //addr.
	{
		cmd[i] =(unsigned char)((addr >> (3-i)*8) & 0xFF );
	}
	SEN_low_F();
	ioSPI_memory_read_F(cmd,retBytes,ret);
	SEN_high_F();
	
	for (i=0; i<retBytes; i++)
	{
		data_in[i] = ret[i]; //*data_in
	}
}

u8 Read_Status_Register()
{
	u8 cmd[4];
	u8 ret[1];

	cmd[0] = 0x05;	//read status register
	
	SEN_low_F();
	ioSPI_read_F(cmd,1,ret);
	SEN_high_F();
	
	return ret[0] ; //*data_in
}

void Write_Status_Register(u8 Byte)
{
	u8 cmd[5];
	
	cmd[0] = 0x50;	//enable write status register
	SEN_low_F();
	ioSPI_write_F(1,cmd);
	SEN_high_F();
	
	cmd[0] = 0x01;	//write status register
	cmd[1] = Byte;	//BP0 & BP1 = 0
	SEN_low_F();
	ioSPI_write_F(2,cmd);
	SEN_high_F();
}

void Write_Disable(void)
{
	u8 cmd[5];
	
	cmd[0] = 0x04;	//write disable
	SEN_low_F();
	ioSPI_write_F(1,cmd);
	SEN_high_F();
}

void Flash_Init(void)
{
	SEN_high_F();
	WP_low_F();
	WP_high_F();
	
	Write_Status_Register(0x00);
}
